#!/usr/bin/python
# -*- coding: UTF-8 -*-

# ---- [ net_io_tools.py ] -----------------------------------------------------
#
# Tools für den AVR-NET-IO in Python 3.2
#
# liest die 4 ADC-Werte und die 4 Eingänge aus
# Setzt die 8 Ausgangspins und liest deren Status
# Schreibt und löscht Zeilen auf dem LCD
#
# --- 08.07.2014
#
# -------------------------------------------------------------------------------

import time
import telnetlib

telnet_host    = '192.100.100.200'
telnet_port    = 50290
telnet_timeout = .2         # bei fehlendem LCD braucht die Antwort 1.65 Sekunden
                            # es zur Wartezeiten bei der Initialisierung
netio          = None

# ------------------------------------------------------------------------------
def Send_Cmd(cmd):
    """ Ein Kommando senden und die Antwort zurückliefern
        Das Ende der Antwort wird am Linefeed "\n" erkannt
    """
    try:
        netio.write(cmd.encode('ascii') + b"\n")
        return netio.read_until(b'\n', telnet_timeout)

    except:
        return b''

# ------------------------------------------------------------------------------
def Get_Version():
    """ Die Software-Version holen und als Liste zeilenweise zurückgeben
        Die Information besteht aus vier Zeilen, die jeweils mit \r\n abgeschlossen sind
        Daher kann das Ende des gesamten Textes nicht anhand \n erkannt werden.
        Liefert eine Liste mit 4 Zeilen zurück, die 4.Zeile ist leer.
    """
    netio.write("VERSION".encode('ascii') + b"\n")
    time.sleep(0.2)             # warten bis die Daten bereitstehen

    t = (netio.read_eager()).decode('ascii')
    return t.split("\r\n")      # in Zeilen aufspalten

# ------------------------------------------------------------------------------
def Get_Adc(channel):
    """ Einen einzelnen ADC-Wert auslesen
        Bei einem Fehler wird -1 zurückgeliefert
    """
    try:
        v = int(Send_Cmd("GETADC " + str(channel)))
    except:
        v = -1
    return v

# ------------------------------------------------------------------------------
def Get_Adcs():
    """ Alle vier ADC-Werte auslesen und als Liste von Int's zurückliefern """
    l = [0,0,0,0]

    for i in [1,2,3,4]:
        l[i-1] = Get_Adc(i)

    return l

# ------------------------------------------------------------------------------
def Get_Port_State():
    """ holt den aktuellen Status der digitalen Ausgänge
        und liefert eine Liste von Int's
    """
    l = [0,0,0,0,0,0,0,0]
    t = (Send_Cmd("GETSTATUS")).decode('ascii')
    t = t[1:10]                  # führendes 'S' und folgendes \r\n' abschneiden

    for i in range(8):
        l[i] = int(t[7-i])       # die Reihenfolge umkehren
    return l

# ------------------------------------------------------------------------------
def Set_Port_State(pin, state):
    """ Setzt den Status eines ditalen Ausgangspins auf High oder Low """
    Send_Cmd("SETPORT " + str(pin) + '.' + str(state))

# ------------------------------------------------------------------------------
def Get_Pin_State():
    """ holt den Status der digitalen Eingangspins
        und lieferte eine Liste von Int's
    """
    l = [0,0,0,0]

    for i in [1,2,3,4]:
        t = (Send_Cmd("GETPORT " + str(i))).decode('ascii')
        l[i-1] = int(t[0])           # nachfolgendes \r\n' abschneiden
    return l

# ------------------------------------------------------------------------------
def Start_Telnet():
    global netio
    success = True

    try:
        netio = telnetlib.Telnet(telnet_host, telnet_port, telnet_timeout)
        x = Get_Version()
        version = "{} - {} - {}".format(x[0], x[1], x[2])
        out = "Host: {} Port: {}".format( telnet_host, telnet_port)
        netio.close()

    except:
        version = "Host: {}:{} nicht gefunden !".format(telnet_host, telnet_port)
        success = False
        out = "Error Host: {} Port: {}".format( telnet_host, telnet_port)

    # Das dritte Element signalisiert das Fehlen eines NET-IO

    return (out, version, success)

# ------------------------------------------------------------------------------
def Close_Telnet():
    netio.close()

# ------------------------------------------------------------------------------
def Open_Telnet():
    netio.open(telnet_host, telnet_port)

# --- LCD ----------------------------------------------------------------------
#
# ------------------------------------------------------------------------------
def Init_LCD():
    """ Der Server liefert einen Bytestring zurück
        wenn im String an der Position 2:5 der Wert "ACK" zu finden ist,
        dann wurde die Initialisierung des LCD's bestätigt.
        Das scheint aber Theorie zu sein.
        Auch wenn kein LCD angschlossen ist, wird ein ACK zurückgegeben.
        Allerdings erst nach ca. 1500ms.
        Bei angeschlossenem LCD folgt das ACK bereits nach 15ms
        Daher wird bei einer Verzögerung von mehr als 200ms davon ausgegangen,
        dass kein LCD vorhanden ist.
        Oder es wird das timeout auf 0.2 gesetzt.
        Wenn die Antwort ""ACK" enthält, dann ist das LCD inialisiert werden.
        In allen anderen Fällen ist wohl kein LCD gefunden worden.

    """
##    t = time.time()
    #print("lcdinit: ", end = "")
    ans = (Send_Cmd("INITLCD")).decode('ascii')
    return (ans[:3] == "ACK")

##    if (time.time() - t) < 0.2:
##        return True
##    else:
##        return False



# -------------------------------------------------------------------------------
def Write_LCD(zeile, textzeile):
    """ Schreibt eine Zeile, Zeile [1,2], maximale Zeichenzahl 16,
        kürzer Zeichenfolgen werden mit Leerzeichen auf Länge 16 erweitert:
        dadurch wird das Zeilenende automatisch gelöscht.
        Zeichen mit Code > 127 werden durch '$' ersetzt,
    """
    txt = ""
    for i in textzeile:             # Zeichen > 127 durch '$' ersetzen
        if ord(i) < 128:
            txt = txt + i
        else:
            txt = txt + '$'

    if len(txt) < 16:               # wenn weniger als 16 Zeichen,
        txt = txt.ljust(16)         # dann rechts mit ' ' auffüllen
##    elif len(txt) > 16:             # wenn länger als 16 Zeichen
##        txt = txt[0:16]             # dann auf 16 Zeichen kappen

    ans = (Send_Cmd("WRITELCD " + str(zeile) + "." + txt)).decode('ascii')
##    return(ans[:3] == "ACK")


# ------------------------------------------------------------------------------
def Clear_LCD(zeile = 0):
    """ löscht eine Zeile (wenn der 2. Paramter angegeben wird als Zeile[1,2])
        bei fehlendem 2. Parameter wird das gesamte Display gelöscht
    """
    if zeile == 0:
        ans = (Send_Cmd("CLEARLCD")).decode('ascii')
    else:
        ans = (Send_Cmd("CLEARLCD " + str(zeile))).decode('ascii')

##    return(ans[:3] == "ACK")

# --- [ eof ] ------------------------------------------------------------------




